use mlua::{Value, FromLua, Table};
use uuid::Uuid;

use crate::{Player, ClientInfo, JsonValue};

use super::UuidUD;

pub enum PluginEvent {
	Kick(Uuid, Option<String>),
	Chat(Uuid, JsonValue, JsonValue),
	BroadcastChat(JsonValue, JsonValue),
	System(Uuid, JsonValue, bool),
	BroadcastSystem(JsonValue, bool),
}

impl<'lua> FromLua<'lua> for PluginEvent {
    fn from_lua(lua_value: Value<'lua>, lua: &'lua mlua::Lua) -> mlua::Result<Self> {
		let table = Table::from_lua(lua_value, lua)?;
		let ty: Box<str> = table.get("type")?;
		match ty.as_ref() {
			"kick" => Ok(Self::Kick(table.get::<_, UuidUD>("uuid")?.0, table.get("msg")?)),
			_ => Err(mlua::Error::RuntimeError(format!("unknown event type {ty}"))),
		}
    }
}

pub enum ServerEvent {
	Join(Player, ClientInfo),
	Leave(Player),
	Command(Player, String),
	Message(Player, String),
}
